"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var of_1 = require("rxjs/observable/of");
var combineLatest_1 = require("rxjs/observable/combineLatest");
function extractRoutes(parent, routeProperty) {
    var routes = [];
    while (parent) {
        routes.push(parent[routeProperty]);
        parent = parent.parent;
    }
    return routes;
}
function extractValues(routes, args, config, cb) {
    var stream$ = combineLatest_1.combineLatest.apply(void 0, [of_1.of(null)].concat(routes, [function () {
            var routeValues = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                routeValues[_i] = arguments[_i];
            }
            var values = routeValues.reduce(function (obj, route) {
                args.forEach(function (arg) {
                    if (route && route[arg] !== undefined) {
                        obj[arg] = route[arg];
                    }
                });
                return obj;
            }, {});
            return args.length === 1 ? values[args[0]] : values;
        }]));
    if (config.observable === false) {
        stream$.subscribe(cb);
    }
    else {
        cb(stream$);
    }
}
function routeDecoratorFactory(routeProperty, args) {
    var config = (typeof args[args.length - 1] === 'object' ? args.pop() : {});
    return function (target, key) {
        var ngOnInit = target.ngOnInit;
        if (!args.length) {
            args = [key.replace(/\$$/, '')];
        }
        if (!target.ngOnInit) {
            throw (new Error(target.constructor.name + " uses the " + routeProperty + " @decorator without implementing 'ngOnInit'"));
        }
        target.ngOnInit = function () {
            var _this = this;
            if (!this.route) {
                throw (new Error(target.constructor.name + " uses the " + routeProperty + " @decorator without a 'route' property"));
            }
            var routes = routeProperty === 'queryParams' ? [this.route.queryParams] : extractRoutes(this.route, routeProperty);
            extractValues(routes, args, config, function (values) {
                _this[key] = values;
            });
            this.ngOnInit = ngOnInit;
            if (ngOnInit) {
                this.ngOnInit();
            }
        };
    };
}
function RouteData() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return routeDecoratorFactory('data', args);
}
exports.RouteData = RouteData;
function RouteParams() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return routeDecoratorFactory('params', args);
}
exports.RouteParams = RouteParams;
function RouteQueryParams() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return routeDecoratorFactory('queryParams', args);
}
exports.RouteQueryParams = RouteQueryParams;
